@@ -9,6 +9,8 @@
  *          iBIZ Technology Corp,  SkyLynx / Inficad Communications
  *          2/12/2000 & 7/18/2000
  *
+ *       Jeffrey Wheat <jeff@cetlink.net> - 10/01/2002
+ *       - Fixed to build with php-4.2.3
  *
  * See README, INSTALL, and USAGE files for more details.
  *
@@ -18,6 +20,7 @@
 
 #include "php.h"
 #include "rrd.h"
+#include "php_config.h"
 #include "php_rrdtool.h"
 
 #if HAVE_RRDTOOL
@@ -34,6 +37,7 @@ function_entry rrdtool_functions[] = {
 };
 
 zend_module_entry rrdtool_module_entry = {
+	STANDARD_MODULE_HEADER,
 	"RRDTool",
 	rrdtool_functions,
 	NULL,
@@ -41,6 +45,7 @@ zend_module_entry rrdtool_module_entry = {
 	NULL,
 	NULL,
 	PHP_MINFO(rrdtool),
+	NO_VERSION_YET,
 	STANDARD_MODULE_PROPERTIES,
 };
 
@@ -366,7 +371,7 @@ PHP_FUNCTION(rrd_fetch)
 	pval *p_start, *p_end, *p_step, *p_ds_cnt;
 	HashTable *args_arr;
 	zval *p_ds_namv, *p_data;
-	int i, argc;
+	int i, j, argc;
 	time_t start, end;
 	unsigned long step, ds_cnt;
 	char **argv, **ds_namv; 
@@ -445,7 +450,8 @@ PHP_FUNCTION(rrd_fetch)
 				datap = data;
  
 				for (i = start; i <= end; i += step)
-					add_next_index_double(p_data, *(datap++));
+					for (j = 0; j < ds_cnt; j++)
+						add_next_index_double(p_data, *(datap++));
  
 				free(data);
 			}
