@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.48  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * rrd_cgi.c  RRD Web Page Generator
  *****************************************************************************/
@@ -433,7 +433,11 @@ char* rrdgetenv(long argc, char **args) {
 	if (envvar) {
 		return stralloc(envvar);
 	} else {
+#ifdef WIN32
+		_snprintf(buf, sizeof(buf), "[ERROR:_getenv_'%s'_failed", args[0]);
+#else
 		snprintf(buf, sizeof(buf), "[ERROR:_getenv_'%s'_failed", args[0]);
+#endif		
 		return stralloc(buf);
 	}
 }
@@ -449,7 +453,11 @@ char* rrdgetvar(long argc, char **args) {
 	if (value) {
 		return stralloc(value);
 	} else {
+#ifdef WIN32
+		_snprintf(buf, sizeof(buf), "[ERROR:_getvar_'%s'_failed", args[0]);
+#else
 		snprintf(buf, sizeof(buf), "[ERROR:_getvar_'%s'_failed", args[0]);
+#endif
 		return stralloc(buf);
 	}
 }
