@@ -1,11 +1,16 @@
 
 /****************************************************************************
- * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.48  Copyright Tobias Oetiker, 1997 - 2000
  ****************************************************************************
  * rrd__graph.c  make creates ne rrds
  ****************************************************************************/
 
 #include "rrd_tool.h"
+
+#ifdef WIN32
+#include <locale.h>
+#endif
+
 #include "rrd_graph.h"
 #include <gd.h>
 
@@ -39,15 +44,6 @@
 #define LargeFont gdLucidaBold12
 #endif
 
-/* #define DEBUG */
-
-#ifdef DEBUG
-# define DPRINT(x)    (void)(printf x, printf("\n"))
-#else
-# define DPRINT(x)
-#endif
-
-
 xlab_t xlab[] = {
     {0,        TMT_SECOND,30, TMT_MINUTE,5,  TMT_MINUTE,5,         0,"%H:%M"},
     {2,        TMT_MINUTE,1,  TMT_MINUTE,5,  TMT_MINUTE,5,         0,"%H:%M"},
@@ -56,7 +52,7 @@ xlab_t xlab[] = {
     {30,       TMT_MINUTE,10, TMT_HOUR,1,    TMT_HOUR,1,           0,"%H:%M"},
     {60,       TMT_MINUTE,30, TMT_HOUR,2,    TMT_HOUR,2,           0,"%H:%M"},
     {180,      TMT_HOUR,1,    TMT_HOUR,6,    TMT_HOUR,6,           0,"%H:%M"},
-    /*{300,      TMT_HOUR,3,    TMT_HOUR,12,   TMT_HOUR,12,    12*3600,"%a %p"},  this looks silly*/
+ /* {300,      TMT_HOUR,3,    TMT_HOUR,12,   TMT_HOUR,12,    12*3600,"%a %p"},  this looks silly */
     {600,      TMT_HOUR,6,    TMT_DAY,1,     TMT_DAY,1,      24*3600,"%a"},
     {1800,     TMT_HOUR,12,   TMT_DAY,1,     TMT_DAY,2,      24*3600,"%a"},
     {3600,     TMT_DAY,1,     TMT_WEEK,1,     TMT_WEEK,1,    7*24*3600,"Week %V"},
@@ -353,7 +349,7 @@ si_unit(
     im->magfact = pow((double)im->base , digits);
 
 #ifdef DEBUG
-    printf("digits %6.3f  im->magfact %6.3f\n",digits,im->magfact);
+    fprintf(stderr,"digits %6.3f  im->magfact %6.3f\n",digits,im->magfact);
 #endif
 
     if ( ((digits+symbcenter) < sizeof(symbol)) &&
@@ -384,16 +380,42 @@ expand_range(image_desc_t *im)
 
     
 #ifdef DEBUG
-    printf("Min: %6.2f Max: %6.2f MagFactor: %6.2f\n",
+    fprintf(stderr,"Min: %6.2f Max: %6.2f MagFactor: %6.2f\n",
 	   im->minval,im->maxval,im->magfact);
 #endif
 
     if (isnan(im->ygridstep)){
 	if(im->extra_flags & ALTYMRTG) { /* mrtg */
- int  decimals = ceil(log10(max(fabs(im->maxval),fabs(im->minval))));
- double scaledstep = ceil(25 * pow((double)10,-decimals) * im->maxval) * pow((double)10,decimals - 2);
- im->minval = 0;
- im->maxval = 4 * scaledstep;
+           im->decimals = ceil(log10(max(fabs(im->maxval),fabs(im->minval))));
+           im->quadrant = 0;
+           if (im->minval < 0) {
+               im->quadrant = 2;
+               if (im->maxval <= 0)
+                   im->quadrant = 4;
+           }
+           switch (im->quadrant) {
+           case 2:
+               im->scaledstep = ceil(50 * pow((double)10,-(im->decimals)) * max(fabs(im->maxval),fabs(im->minval))) * pow((double)10,im->decimals - 2);
+               scaled_min = -2 * im->scaledstep;
+               scaled_max = 2 * im->scaledstep;
+               break;
+           case 4:
+               im->scaledstep = ceil(25 * pow((double)10,-(im->decimals)) * fabs(im->minval)) * pow((double)10,im->decimals - 2);
+               scaled_min = -4 * im->scaledstep;
+               scaled_max = 0;
+               break;
+           default: /* quadrant 0 */
+               im->scaledstep = ceil(25 * pow((double)10,-(im->decimals)) * im->maxval) * pow((double)10,im->decimals - 2);
+               scaled_min = 0;
+               scaled_max = 4 * im->scaledstep;
+               break;
+           }
+#ifdef DEBUG
+           fprintf(stderr,"[%d] %s Min: %6.2f Max: %6.2f MagFactor: %6.2f, decimals: %d scaledstep: %6.2f -> scaled Min: %6.2f Max %6.2f\n",
+                   getpid(),im->graphfile,im->minval,im->maxval,im->magfact,im->decimals,im->scaledstep,scaled_min,scaled_max);
+#endif
+           im->minval = scaled_min;
+           im->maxval = scaled_max;
 	} else
 	if(im->extra_flags & ALTAUTOSCALE) {
 	    /* measure the amplitude of the function. Make sure that
@@ -408,7 +430,7 @@ expand_range(image_desc_t *im)
 	      if (delt < fact) {
 		adj = (fact - delt) * 0.55;
 #ifdef DEBUG
-	      printf("Min: %6.2f Max: %6.2f delt: %6.2f fact: %6.2f adj: %6.2f\n", im->minval, im->maxval, delt, fact, adj);
+	      fprintf(stderr,"Min: %6.2f Max: %6.2f delt: %6.2f fact: %6.2f adj: %6.2f\n", im->minval, im->maxval, delt, fact, adj);
 #endif
 	      }
 	      im->minval -= adj;
@@ -1578,9 +1600,12 @@ print_calc(image_desc_t *im, char ***prdata)
 	    for(ii=im->gdes[vidx].ds+im->gdes[vidx].ds_cnt;
 		ii < max_ii+im->gdes[vidx].ds_cnt;
 		ii+=im->gdes[vidx].ds_cnt){
-		 if (! finite(im->gdes[vidx].data[ii]))
+#ifdef DEBUG
+                fprintf(stderr,"[%d] %s value: %6.2f\n",getpid(),im->graphfile,im->gdes[vidx].data[ii]);
+#endif
+  	        if (! finite(im->gdes[vidx].data[ii]))
 		     continue;
-		 if (isnan(printval)){
+		if (isnan(printval)){
 		     printval = im->gdes[vidx].data[ii];
 		     validsteps++;
 		    continue;
@@ -1688,7 +1713,7 @@ leg_place(image_desc_t *im)
     char  prt_fctn; /*special printfunctions */
     int  *legspace;
 
-    if( !(im->extra_flags & NOLEGEND) ) {
+    if( !(im->extra_flags & NOLEGEND) & !(im->extra_flags & ONLY_GRAPH) ) {
       if ((legspace = malloc(im->gdes_c*sizeof(int)))==NULL){
          rrd_set_error("malloc for legspace");
          return -1;
@@ -1697,13 +1722,15 @@ leg_place(image_desc_t *im)
     for(i=0;i<im->gdes_c;i++){
 	fill_last = fill;
 
-	if (im->gdes[i].gf == GF_HRULE &&
-	    (im->gdes[i].yrule < im->minval || im->gdes[i].yrule > im->maxval))
-	    im->gdes[i].legend[0] = '\0';
+	if( !(im->extra_flags & FORCE_RULES_LEGEND) ) {
+	    if (im->gdes[i].gf == GF_HRULE &&
+		(im->gdes[i].yrule < im->minval || im->gdes[i].yrule > im->maxval))
+		im->gdes[i].legend[0] = '\0';
 
-	if (im->gdes[i].gf == GF_VRULE &&
-	    (im->gdes[i].xrule < im->start || im->gdes[i].xrule > im->end))
-	    im->gdes[i].legend[0] = '\0';
+	    if (im->gdes[i].gf == GF_VRULE &&
+		(im->gdes[i].xrule < im->start || im->gdes[i].xrule > im->end))
+		im->gdes[i].legend[0] = '\0';
+	}
 
 	leg_cc = strlen(im->gdes[i].legend);
 	
@@ -1966,10 +1993,8 @@ int
 horizontal_mrtg_grid(gdImagePtr gif, image_desc_t *im)
 {
  int      i, xLeft, xRight, y;
- double   scaledstep;
  char     graph_label[100];
  int      styleMajor[2];
- int      decimals;
  char     labfmt[64];
 
  /* does the scale of this graph make it impossible to put lines
@@ -1983,21 +2008,25 @@ horizontal_mrtg_grid(gdImagePtr gif, image_desc_t *im)
  xLeft = im->xorigin;
  xRight = im->xorigin + im->xsize;
 
- decimals = ceil(log10(max(fabs(im->maxval),fabs(im->minval))));
- scaledstep = ceil(25 * pow((double)10,-decimals) * im->maxval) * pow((double)10,decimals - 2) / im->magfact;
-
- sprintf(labfmt,"%%4.%df",1 - (scaledstep > (double)10 || ceil(scaledstep) == scaledstep));
+ if (im->scaledstep / im->magfact * max(abs(im->quadrant),abs(4 - im->quadrant)) <= (double)1)
+    strcpy(labfmt,"%5.2f");
+ else
+    sprintf(labfmt,"%%4.%df",1 - (im->scaledstep / im->magfact > (double)10 || ceil(im->scaledstep / im->magfact) == im->scaledstep /im->magfact));
  if (im->symbol != ' ' || im->unit)
-  strcat(labfmt," ");
+    strcat(labfmt," ");
  if (im->symbol != ' ')
-  sprintf(labfmt + strlen(labfmt),"%c",im->symbol);
+    sprintf(labfmt + strlen(labfmt),"%c",im->symbol);
  if (im->unit)
-  strcat(labfmt,im->unit);
+    strcat(labfmt,im->unit);
 
  for (i = 0; i <= 4; i++)
   {y = im->yorigin - im->ysize * i / 4;
    if (y >= im->yorigin-im->ysize && y <= im->yorigin)
-    {sprintf(graph_label,labfmt,scaledstep * i);
+    {sprintf(graph_label,labfmt,im->scaledstep / im->magfact * (i - im->quadrant));
+#ifdef DEBUG
+     fprintf(stderr,"[%d] scaledstep * (i - quadrant): %6.2f format '%s' label: '%s'\n",
+             getpid(),im->scaledstep * (i - im->quadrant),labfmt,graph_label);
+#endif
      gdImageString(gif,SmallFont,(xLeft - (strlen(graph_label) * SmallFont->w) - 7),
                                  y - SmallFont->h / 2 + 1,
                                  (unsigned char *)graph_label,graph_col[GRC_FONT].i);
@@ -2316,7 +2345,7 @@ grid_paint(
 		    (unsigned char *)im->title, graph_col[GRC_FONT].i);
     
     /* graph labels */
-    if( !(im->extra_flags & NOLEGEND) ) {
+    if( !(im->extra_flags & NOLEGEND) & !(im->extra_flags & ONLY_GRAPH) ) {
       for(i=0;i<im->gdes_c;i++){
 	if(im->gdes[i].legend[0] =='\0')
 	    continue;
@@ -2515,21 +2544,39 @@ graph_paint(image_desc_t *im, char ***calcpr)
        on the cmdline is the graph area. But we need more as we have
        draw labels and other things outside the graph area */
 
+    if( (im->extra_flags & ONLY_GRAPH) ) {
+        if ( im->ysize > 64 ) {
+            rrd_set_error("height over 32 pixels with --only-graph option is not available");
+	    return -1;
+        }
+        im->xorigin = 0;
+    } else {
+        im->xorigin = 10 + im->unitslength * SmallFont->w;
+    }
 
-    im->xorigin = 10 + im->unitslength * SmallFont->w;
     xtr(im,0);
 
     if(im->ylegend[0] != '\0')
         im->xorigin += SmallFont->h;
 
-    im->yorigin = 14 + im->ysize;
+    if( (im->extra_flags & ONLY_GRAPH) ) {
+        im->yorigin = im->ysize;
+    } else {
+        im->yorigin = 14 + im->ysize;
+    }
+
     ytr(im,DNAN);
 
     if(im->title[0] != '\0')
 	im->yorigin += (LargeFont->h+4);
 
-    im->xgif=20+im->xsize + im->xorigin;
-    im->ygif= im->yorigin+2*SmallFont->h;
+    if( (im->extra_flags & ONLY_GRAPH) ) {
+        im->xgif= im->xsize;
+        im->ygif= im->yorigin;
+    } else {
+        im->xgif=16+im->xsize + im->xorigin;
+        im->ygif= im->yorigin+2*SmallFont->h;
+    }
     
     /* determine where to place the legends onto the graphics.
        and set im->ygif to match space requirements for text */
@@ -2571,7 +2618,7 @@ graph_paint(image_desc_t *im, char ***calcpr)
        graph elements and then drawing them */
 
   if (im->bkg_image)	/* background image added first */
-   copyImage(gif,im->bkg_image,1);
+    copyImage(gif,im->bkg_image,1);
   else
    {
     back[0].x = 0;
@@ -2602,7 +2649,8 @@ graph_paint(image_desc_t *im, char ***calcpr)
     if (im->maxval < 0.0)
 	areazero = im->maxval;
 
-    axis_paint(im,gif);
+    if( !(im->extra_flags & ONLY_GRAPH) ) 
+        axis_paint(im,gif);
 
     for(i=0;i<im->gdes_c;i++){	
         
@@ -2686,7 +2734,8 @@ graph_paint(image_desc_t *im, char ***calcpr)
 	}
     }
     
-    grid_paint(im,gif);
+    if( !(im->extra_flags & ONLY_GRAPH) ) 
+        grid_paint(im,gif); 
 
     /* the RULES are the last thing to paint ... */
     for(i=0;i<im->gdes_c;i++){	
@@ -2898,6 +2947,8 @@ im.unit = 0; /* do not display unit if not set */
 	    {"imgformat",  required_argument, 0,  'a'},
 	    {"lazy",       no_argument,       0,  'z'},
 	    {"no-legend",  no_argument,       0,  'g'},
+            {"only-graph",  no_argument,       0,  'j'},
+      	    {"force-rules-legend",no_argument,0,  'F'},
 	    {"alt-y-grid", no_argument,       0,   'Y'},
             {"no-minor",   no_argument,       0,   'N'},
  	    {"alt-autoscale", no_argument,    0,   'A'},
@@ -2915,7 +2966,7 @@ im.unit = 0; /* do not display unit if not set */
 
 	
 	opt = getopt_long(argc, argv, 
-			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgYAMXL:S:RB:O:U:",
+			  "s:e:x:y:v:w:h:iu:l:rb:oc:t:f:a:zgjFYAMXL:S:RB:O:U:",
 			  long_options, &option_index);
 
 	if (opt == EOF)
@@ -2934,9 +2985,15 @@ im.unit = 0; /* do not display unit if not set */
 	case 'M':
 	    im.extra_flags |= ALTAUTOSCALE_MAX;
 	    break;
+	case 'j':
+	    im.extra_flags |= ONLY_GRAPH;
+	    break;
 	case 'g':
 	    im.extra_flags |= NOLEGEND;
 	    break;
+	case 'F':
+	    im.extra_flags |= FORCE_RULES_LEGEND;
+	    break;
 	case 'X':
 	    im.unitsexponent = atoi(optarg);
 	    break;
