@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2000
+ * RRDtool 1.0.48  Copyright Tobias Oetiker, 1997 - 2000
  *****************************************************************************
  * rrd_restore.c  creates new rrd from data dumped by rrd_dump.c
  *****************************************************************************/
@@ -280,7 +280,14 @@ rrd_write(char *file_name, rrd_t *rrd)
     if (strcmp("-",file_name)==0){
       rrd_file= stdout;
     } else {
-      int fd = open(file_name,O_RDWR|O_CREAT|O_EXCL,0666);
+	int fd = open(file_name,
+#ifdef WIN32
+        O_RDWR|O_BINARY|O_CREAT|O_EXCL
+#else
+        O_RDWR|O_CREAT|O_EXCL
+#endif
+	,0666);
+	
       if (fd == -1 || (rrd_file = fdopen(fd,"wb")) == NULL) {
 	rrd_set_error("creating '%s': %s",file_name,strerror(errno));
         if (fd != -1)
