@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.0.46  Copyright Tobias Oetiker, 1997 - 2001
+ * RRDtool 1.0.48  Copyright Tobias Oetiker, 1997 - 2004
  *****************************************************************************
  * rrd_tool.c  Startup wrapper
  *****************************************************************************/
@@ -20,7 +20,7 @@ void PrintUsage(char *cmd)
 {
 
     char help_main[] =
-	   "RRDtool 1.0.46  Copyright 1997-2001 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
+	   "RRDtool 1.0.48  Copyright 1997-2004 by Tobias Oetiker <tobi@oetiker.ch>\n\n"
 	   "Usage: rrdtool [options] command command_options\n\n";
 
     char help_list[] =
@@ -73,6 +73,8 @@ void PrintUsage(char *cmd)
 	   "\t\t[-u|--upper-limit value] [-z|--lazy]\n"
 	   "\t\t[-l|--lower-limit value] [-r|--rigid]\n"
            "\t\t[-g|--no-legend]\n"
+           "\t\t[-j|--only-graph]\n"
+           "\t\t[-F|--force-rules-legend]\n"
 	   "\t\t[--alt-autoscale]\n"
 	   "\t\t[--alt-autoscale-max]\n"
 	   "\t\t[--units-exponent value]\n"	   
@@ -320,7 +322,7 @@ int HandleInputLine(int argc, char **argv, FILE* out)
 	     strcmp("v", argv[1]) == 0 ||
 	     strcmp("-v", argv[1]) == 0  ||
 	     strcmp("-version", argv[1]) == 0  )
-        printf("RRDtool 1.0.46  Copyright (C) 1997-2001 by Tobias Oetiker <tobi@oetiker.ch>\n");
+        printf("RRDtool 1.0.48  Copyright (C) 1997-2004 by Tobias Oetiker <tobi@oetiker.ch>\n");
     else if (strcmp("restore", argv[1]) == 0)
 	rrd_restore(argc-1, &argv[1]);
     else if (strcmp("resize", argv[1]) == 0)
@@ -336,9 +338,9 @@ int HandleInputLine(int argc, char **argv, FILE* out)
 	char          **ds_namv;
 	if (rrd_fetch(argc-1, &argv[1],&start,&end,&step,&ds_cnt,&ds_namv,&data) != -1) {
 	    datai=data;
-	    printf("           ");
+	    printf("          ");
 	    for (i = 0; i<ds_cnt;i++)
-	        printf("%20s",ds_namv[i]);
+	        printf(" %20s",ds_namv[i]);
 	    printf ("\n\n");
 	    for (i = start; i <= end; i += step){
 	        printf("%10lu:", i);
